// src/response.ts
var ERR_ABORTED = new Error("ERR_ABORTED");
var ERR_TIMEOUT = new Error("ERR_TIMEOUT");
var ERR_NETWORK = new Error("ERR_NETWORK");
var ERR_NOT_FOUND_HANDLER = new Error("ERR_NOT_FOUND_HANDLER");
var ERR_OBSERVE = new Error("ERR_OBSERVE");
var ERR_NOT_SET_ALIAS = new Error("ERR_NOT_SET_ALIAS");
var ERR_UNSUPPORTED_FIELD_TYPE = new Error("ERR_UNSUPPORTED_FIELD_TYPE");
var ERR_INVALID_CLIENT = new Error("ERR_INVALID_CLIENT");
var ERR_NOT_FOUND_GLOBAL_CLIENT = new Error("ERR_NOT_FOUND_GLOBAL_CLIENT");
var ERR_INVALID_HTTP_CONTEXT_TOKEN = new Error("ERR_INVALID_HTTP_CONTEXT_TOKEN");
var ERR_UNKNOWN = new Error("ERR_UNKNOWN");
function __makeResponse(options) {
  const status = options?.status ?? 0;
  const ok = status >= 200 && status < 300;
  const statusText = options?.statusText ?? "";
  const url = options?.url ?? "";
  const headers = options?.headers ?? new Headers;
  const body = options?.body ?? null;
  let error = options?.error;
  if (!error && !ok) {
    let message = `Http failure response for ${url || "(unknown url)"}: ${status}`;
    if (statusText) {
      message += ` - ${statusText}`;
    }
    error = new Error(message);
  }
  return {
    status,
    statusText,
    url,
    headers,
    body,
    error
  };
}

// src/context.ts
var HTTP_CONTEXT_TOKEN = Symbol("HttpContextToken");
function makeHttpContextToken(defaultValue) {
  Object.defineProperty(defaultValue, HTTP_CONTEXT_TOKEN, { value: defaultValue });
  return defaultValue;
}
function isHttpContextToken(value) {
  return typeof value === "function" && HTTP_CONTEXT_TOKEN in value;
}
var HTTP_CONTEXT = Symbol("HttpContext");
function isHttpContext(value) {
  return typeof value === "object" && value !== null && HTTP_CONTEXT in value;
}
function makeHttpContext(entries) {
  const ctx = new Map;
  switch (true) {
    case Array.isArray(entries): {
      for (const [token, value] of entries) {
        if (isHttpContextToken(token)) {
          ctx.set(token, value);
        }
      }
      break;
    }
    case isHttpContext(entries): {
      for (const [token, value] of entries[HTTP_CONTEXT]) {
        ctx.set(token, value);
      }
      break;
    }
  }
  return {
    set(token, value) {
      if (!isHttpContextToken(token)) {
        throw ERR_INVALID_HTTP_CONTEXT_TOKEN;
      }
      ctx.set(token, value);
      return this;
    },
    get(token) {
      if (!isHttpContextToken(token)) {
        throw ERR_INVALID_HTTP_CONTEXT_TOKEN;
      }
      if (!ctx.has(token)) {
        ctx.set(token, token());
      }
      return ctx.get(token);
    },
    del(token) {
      ctx.delete(token);
      return this;
    },
    has(token) {
      return ctx.has(token);
    },
    keys() {
      return ctx.keys();
    },
    get length() {
      return Array.from(ctx.keys()).length;
    },
    [HTTP_CONTEXT]: ctx
  };
}

// src/field.ts
var FIELD = Symbol("field");
function field(defaultValue) {
  const meta = {
    required: typeof defaultValue !== "undefined",
    alias: new Map,
    validators: [],
    asyncValidator: []
  };
  const getter = () => defaultValue;
  Object.defineProperty(getter, FIELD, {
    value: meta
  });
  getter.withJson = (alias) => {
    meta.alias.set(0 /* Json */, alias);
    return getter;
  };
  getter.withForm = (alias) => {
    meta.alias.set(4 /* Form */, alias);
    return getter;
  };
  getter.withQuery = (alias) => {
    meta.alias.set(1 /* Query */, alias);
    return getter;
  };
  getter.withParam = (alias) => {
    meta.alias.set(2 /* Param */, alias);
    return getter;
  };
  getter.withHeader = (alias) => {
    meta.alias.set(3 /* Header */, alias);
    return getter;
  };
  getter.withUrlForm = (alias) => {
    meta.alias.set(6 /* UrlForm */, alias);
    return getter;
  };
  getter.withBody = () => {
    meta.alias.set(5 /* Body */, undefined);
    return getter;
  };
  getter.withValidators = (...fn) => {
    meta.validators = fn;
    return getter;
  };
  getter.withAsyncValidators = (...fn) => {
    meta.asyncValidator = fn;
    return getter;
  };
  return getter;
}
function __getFieldMetadata(field2) {
  return field2[FIELD];
}
async function doValid(validators, value) {
  await Promise.all(validators.map((validator) => validator(value)));
}
function isField(value) {
  return !!value && typeof value === "function" && FIELD in value;
}
function isFieldGroup(value) {
  return !!value && typeof value === "object" && Object.entries(value).every(([, v]) => isField(v));
}

// src/handler/handler.ts
var globalHttpHandler = undefined;
var getGlobalHttpHandler = () => globalHttpHandler;

// src/interceptor/interceptor.ts
function makeInterceptorChain(interceptors) {
  return interceptors.reduceRight((fn, interceptor) => {
    return (initReq, finalHandlerFn) => interceptor(initReq, (req) => fn(req, finalHandlerFn));
  }, (req, fn) => fn(req));
}

// src/request.ts
function __serializeBody(body) {
  switch (true) {
    case body instanceof FormData:
    case body instanceof Blob:
    case body instanceof ArrayBuffer:
    case body instanceof URLSearchParams:
    case typeof body === "string":
      return body;
    case typeof body === "object":
    case typeof body === "boolean":
    case typeof body === "number":
    case Array.isArray(body):
      return JSON.stringify(body);
    default:
      return null;
  }
}
function __detectContentTypeHeader(body) {
  switch (true) {
    case body instanceof FormData:
    case body instanceof URLSearchParams:
      return null;
    case body instanceof ArrayBuffer:
      return "application/octet-stream";
    case body instanceof Blob:
      return body.type || "application/octet-stream";
    case typeof body === "string":
      return "text/plain";
    case (typeof body === "object" && body !== null):
    case typeof body === "number":
    case typeof body === "boolean":
      return "application/json";
    default:
      return null;
  }
}
function __buildFieldDefaultValue(input) {
  if (isFieldGroup(input)) {
    const obj = {};
    for (const [key, field2] of Object.entries(input)) {
      obj[key] = field2();
    }
    return obj;
  }
  return;
}
function defineRequest(...args) {
  let method;
  let endpoint;
  let requiredInput = false;
  let field2;
  let interceptors = [];
  let responseType = "json";
  let context;
  let withCredentials = false;
  let validators = [];
  let transformResponse;
  switch (args.length) {
    case 1: {
      method = "GET";
      endpoint = args[0];
      break;
    }
    case 2: {
      method = args[0];
      endpoint = args[1];
      break;
    }
    default:
      throw new Error("Invalid arguments");
  }
  const fn = (options) => {
    const timeout = options?.timeout || 0;
    const client = options?.client || getGlobalClient();
    const clientOptions = getClientConfig(client);
    const handler = options?.handler || clientOptions?.handler || getGlobalHttpHandler();
    if (!handler) {
      throw ERR_NOT_FOUND_HANDLER;
    }
    let uploadProgress;
    let downloadProgress;
    let abortController;
    const abortSignal = [];
    if (options?.abort) {
      abortSignal.push(options.abort);
    } else {
      abortController = new AbortController;
      abortSignal.push(abortController.signal);
    }
    if (timeout > 0) {
      abortSignal.push(AbortSignal.timeout(timeout));
    }
    const cancel = () => {
      abortController?.abort();
    };
    const getInitValue = () => __buildFieldDefaultValue(field2);
    const setUploadProgress = (fn2) => {
      uploadProgress = fn2;
    };
    const setDownloadProgress = (fn2) => {
      downloadProgress = fn2;
    };
    const doRequest = async (...args2) => {
      try {
        let input;
        if (requiredInput && args2.length === 1) {
          input = args2[0];
        }
        if (requiredInput && !input) {
          throw new Error(`Because the request has input, the argument must be the input value`);
        }
        if (validators.length > 0) {
          for (const fn2 of validators) {
            fn2(input);
          }
        }
        const req = {
          host: clientOptions?.host,
          method,
          endpoint,
          queryParams: new URLSearchParams,
          headers: new Headers,
          body: undefined,
          withCredentials,
          responseType,
          context: context || makeHttpContext(),
          uploadProgress,
          downloadProgress,
          timeout,
          abort: AbortSignal.any(abortSignal)
        };
        if (requiredInput && field2) {
          await __fillRequestFromField(req, field2, input);
        }
        const chain = makeInterceptorChain([...clientOptions.interceptors, ...interceptors]);
        let res = await chain(req, handler);
        if (typeof transformResponse === "function") {
          res = __makeResponse({
            ...res,
            body: transformResponse(res)
          });
        }
        return res;
      } catch (error) {
        return __makeResponse({ error });
      }
    };
    return {
      doRequest,
      cancel,
      getInitValue,
      setUploadProgress,
      setDownloadProgress
    };
  };
  fn.withField = (value) => {
    field2 = value;
    requiredInput = true;
    return fn;
  };
  fn.withInterceptors = (value) => {
    interceptors = value;
    return fn;
  };
  fn.withContext = (value) => {
    context = value;
    return fn;
  };
  fn.withCredentials = (value) => {
    withCredentials = value;
    return fn;
  };
  fn.withValidators = (...fns) => {
    validators = fns;
    return fn;
  };
  fn.withTransformResponse = (value) => {
    transformResponse = value;
    return fn;
  };
  fn.withResponseType = (value) => {
    responseType = value;
    return fn;
  };
  return fn;
}
function __fillUrl(endpoint, params) {
  return endpoint.replace(/:([^\/]+)/g, (_, p1) => {
    return params.get(p1) ?? "undefined";
  });
}
async function __fillRequestFromField(request, fieldOrFieldGroup, input) {
  function inputIsObject(input2) {
    return typeof input2 === "object" && input2 !== null;
  }
  function serializeToString(value) {
    switch (true) {
      case Array.isArray(value):
      case (typeof value === "object" && value !== null):
        return JSON.stringify(value);
      case (typeof value === "object" && value === null):
      case typeof value === "boolean":
      case typeof value === "number":
      case typeof value === "string":
      case typeof value === "undefined":
        return String(value);
      default:
        throw new Error(`Unsupported value type: ${typeof value}`);
    }
  }
  function appendValue(sp, key, value) {
    if (Array.isArray(value)) {
      for (const v of value) {
        sp.append(key, serializeToString(v));
      }
      return sp;
    }
    if (typeof value !== "undefined") {
      sp.set(key, serializeToString(value));
    }
    return sp;
  }
  function setValue(sp, key, value) {
    if (Array.isArray(value)) {
      for (const v of value) {
        setValue(sp, key, serializeToString(v));
      }
      return sp;
    }
    if (typeof value !== "undefined") {
      sp.set(key, serializeToString(value));
    }
    return sp;
  }
  function getValidValue(value, defaultValue) {
    switch (true) {
      case typeof value === "undefined":
        return defaultValue;
      case (typeof value === "object" && value === null):
        return value;
      default:
        return value;
    }
  }
  if (isField(fieldOrFieldGroup)) {
    let params;
    let queryParams;
    let headers;
    let body = undefined;
    const validValue = getValidValue(input, fieldOrFieldGroup());
    const meta = __getFieldMetadata(fieldOrFieldGroup);
    await doValid([...meta.validators, ...meta.asyncValidator], validValue);
    for (const [type, aliasName] of meta.alias) {
      switch (type) {
        case 1 /* Query */: {
          if (!aliasName) {
            throw ERR_NOT_SET_ALIAS;
          }
          queryParams = appendValue(new URLSearchParams, aliasName, validValue);
          break;
        }
        case 2 /* Param */: {
          if (!aliasName) {
            throw ERR_NOT_SET_ALIAS;
          }
          params = setValue(new Map, aliasName, validValue);
          break;
        }
        case 3 /* Header */: {
          if (!aliasName) {
            throw ERR_NOT_SET_ALIAS;
          }
          headers = appendValue(new Headers, aliasName, validValue);
          break;
        }
        case 4 /* Form */: {
          if (!aliasName) {
            throw ERR_NOT_SET_ALIAS;
          }
          body = appendValue(new FormData, aliasName, validValue);
          break;
        }
        case 6 /* UrlForm */: {
          if (!aliasName) {
            throw ERR_NOT_SET_ALIAS;
          }
          body = appendValue(new URLSearchParams, aliasName, validValue);
          break;
        }
        case 0 /* Json */:
        case 5 /* Body */:
          body = validValue;
          break;
      }
    }
    if (params) {
      request.endpoint = __fillUrl(request.endpoint, params);
    }
    request.queryParams = queryParams;
    request.headers = headers;
    request.body = body;
    return;
  }
  if (isFieldGroup(fieldOrFieldGroup) && inputIsObject(input)) {
    let params = new Map;
    let queryParams = new URLSearchParams;
    let headers = new Headers;
    let urlForm = new URLSearchParams;
    let formData = new FormData;
    const json = {};
    let body = undefined;
    let lastBodyField;
    for (const [propertyKey, field2] of Object.entries(fieldOrFieldGroup)) {
      const meta = __getFieldMetadata(field2);
      for (const [type, aliasName] of meta.alias) {
        const valueKey = aliasName || propertyKey;
        const value = getValidValue(input[propertyKey], field2());
        await doValid([...meta.validators, ...meta.asyncValidator], value);
        switch (type) {
          case 0 /* Json */: {
            Object.defineProperty(json, valueKey, {
              value,
              writable: true,
              enumerable: true
            });
            lastBodyField = 0 /* Json */;
            break;
          }
          case 1 /* Query */:
            queryParams = appendValue(queryParams, valueKey, value);
            break;
          case 2 /* Param */:
            params = setValue(params, valueKey, value);
            break;
          case 3 /* Header */:
            headers = appendValue(headers, valueKey, value);
            break;
          case 4 /* Form */: {
            formData = appendValue(formData, valueKey, value);
            lastBodyField = 4 /* Form */;
            break;
          }
          case 6 /* UrlForm */: {
            urlForm = appendValue(urlForm, valueKey, value);
            lastBodyField = 6 /* UrlForm */;
            break;
          }
          case 5 /* Body */: {
            body = value;
            lastBodyField = 5 /* Body */;
            break;
          }
        }
      }
    }
    request.endpoint = __fillUrl(request.endpoint, params);
    request.queryParams = queryParams;
    request.headers = headers;
    switch (lastBodyField) {
      case 5 /* Body */:
        request.body = body;
        break;
      case 4 /* Form */:
        request.body = formData;
        break;
      case 6 /* UrlForm */:
        request.body = urlForm;
        break;
      case 0 /* Json */:
        request.body = json;
        break;
    }
    return;
  }
  throw ERR_UNSUPPORTED_FIELD_TYPE;
}

// src/handler/util.ts
function __getContentLength(headers) {
  const value = headers.get("Content-Length");
  if (!value) {
    return 0;
  }
  const num = Number(value);
  if (isNaN(num)) {
    return 0;
  }
  return num;
}
function __getContentType(headers) {
  return headers.get("Content-Type") || "";
}
function __parseBody(params) {
  const { request, content, contentType } = params;
  const responseType = request.responseType;
  switch (responseType) {
    case "json": {
      const text = new TextDecoder().decode(content);
      if (text === "") {
        return null;
      }
      return JSON.parse(text);
    }
    case "text":
      return new TextDecoder().decode(content);
    case "blob": {
      return new Blob([content], { type: contentType });
    }
    case "arraybuffer":
      return content.buffer;
    default:
      return null;
  }
}
function __concatChunks(chunks, totalLength) {
  const chunksAll = new Uint8Array(totalLength);
  let position = 0;
  for (const chunk of chunks) {
    chunksAll.set(chunk, position);
    position += chunk.length;
  }
  return chunksAll;
}

// src/handler/fetch/fetch.ts
function __createRequest(request) {
  const url = new URL(request.endpoint, request.host);
  if (request.queryParams) {
    url.search = request.queryParams.toString();
  }
  const headers = request.headers ?? new Headers;
  if (!headers.has("Content-Type")) {
    const detectedType = __detectContentTypeHeader(request.body);
    if (detectedType) {
      headers.set("Content-Type", detectedType);
    }
  }
  if (!headers.has("Accept")) {
    headers.set("Accept", "application/json, text/plain, */*");
  }
  const credentials = request.withCredentials ? "include" : undefined;
  return new Request(url, {
    headers,
    method: request.method,
    body: __serializeBody(request.body),
    signal: request.abort,
    credentials
  });
}
async function fetchHandler(httpRequest) {
  const downloadProgress = httpRequest.downloadProgress;
  const request = __createRequest(httpRequest);
  const abortSignal = httpRequest.abort;
  let response;
  try {
    response = await globalThis.fetch(request);
  } catch (error) {
    if (abortSignal?.aborted && abortSignal.reason instanceof Error) {
      switch (true) {
        case abortSignal.reason.name === "AbortError":
          return __makeResponse({ error: ERR_ABORTED });
        case abortSignal.reason.name === "TimeoutError":
          return __makeResponse({ error: ERR_TIMEOUT });
      }
    }
    return __makeResponse({ error });
  }
  const { headers, status, statusText, url } = response;
  const contentLength = __getContentLength(headers);
  const contentType = __getContentType(headers);
  let body = null;
  if (response.body) {
    const chunks = [];
    const reader = response.body.getReader();
    let receivedLength = 0;
    while (true) {
      const { done, value } = await reader.read();
      if (done) {
        break;
      }
      chunks.push(value);
      receivedLength += value.length;
      downloadProgress?.({
        lengthComputable: contentLength > 0,
        loaded: receivedLength,
        total: contentLength
      });
    }
    const chunksAll = __concatChunks(chunks, receivedLength);
    try {
      body = __parseBody({
        request: httpRequest,
        content: chunksAll,
        contentType
      });
    } catch (error) {
      return __makeResponse({
        error,
        status,
        statusText,
        headers,
        url
      });
    }
  }
  return __makeResponse({
    status,
    statusText,
    headers,
    url,
    body
  });
}
// src/client.ts
var CLIENT = Symbol("Client");
var globalClient;
function isClient(value) {
  return typeof value === "object" && value !== null && CLIENT in value;
}
function getClientConfig(client) {
  if (!isClient(client)) {
    throw ERR_INVALID_CLIENT;
  }
  return client[CLIENT];
}
function createClient(options) {
  const conf = {
    host: options.host,
    interceptors: options.interceptors ?? [],
    handler: options.handler || fetchHandler
  };
  return {
    [CLIENT]: conf
  };
}
function cloneClient(client, options) {
  const preConf = getClientConfig(client);
  const conf = {
    ...preConf,
    ...options
  };
  return {
    [CLIENT]: conf
  };
}
function getGlobalClient() {
  if (!globalClient) {
    throw ERR_NOT_FOUND_GLOBAL_CLIENT;
  }
  return globalClient;
}
function setGlobalClient(client) {
  globalClient = client;
}
function restGlobalClient() {
  globalClient = undefined;
}
function createGlobalClient(options) {
  globalClient = createClient(options);
}
// src/interceptor/authorization/basic_auth_interceptor.ts
function basicAuthInterceptor(fn, options) {
  let encode = options?.encode;
  if (!encode) {
    const btoa = globalThis.btoa;
    if (typeof btoa !== "function") {
      throw new Error("BasicAuthInterceptor is not supported in this environment");
    }
    encode = (data) => btoa(`${data.username}:${data.password}`);
  }
  return (req, next) => {
    const headers = req.headers || new Headers;
    headers.set("Authorization", `Basic ${encode(fn())}`);
    req.headers = headers;
    return next(req);
  };
}
// src/validator.ts
var required = (value) => {
  if (!value) {
    return new Error("required");
  }
  return null;
};
var requiredTrue = (value) => {
  if (typeof value !== "boolean" || !value) {
    return new Error("requiredTrue");
  }
  return null;
};
var minLength = (min) => {
  return (value) => {
    if (typeof value !== "string") {
      return new Error("invalid string");
    }
    if (value.length < min) {
      return new Error(`minLength ${min}`);
    }
    return null;
  };
};
var maxLength = (max) => {
  return (value) => {
    if (typeof value !== "string") {
      return new Error("invalid string");
    }
    if (value.length > max) {
      return new Error(`maxLength ${max}`);
    }
    return null;
  };
};
var min = (min2) => {
  return (value) => {
    if (typeof value !== "number" || isNaN(value)) {
      return new Error("invalid number");
    }
    if (value < min2) {
      return new Error(`minLength ${min2}`);
    }
    return null;
  };
};
var max = (max2) => {
  return (value) => {
    if (typeof value !== "number" || isNaN(value)) {
      return new Error("invalid number");
    }
    if (value > max2) {
      return new Error(`maxLength ${max2}`);
    }
    return null;
  };
};
var pattern = (pattern2) => {
  return (value) => {
    if (typeof value !== "string") {
      return new Error("invalid string");
    }
    if (!pattern2.test(value)) {
      return new Error("pattern");
    }
    return null;
  };
};
export {
  setGlobalClient,
  restGlobalClient,
  requiredTrue,
  required,
  pattern,
  minLength,
  min,
  maxLength,
  max,
  makeHttpContextToken,
  makeHttpContext,
  isHttpContextToken,
  isFieldGroup,
  isField,
  isClient,
  getGlobalClient,
  field,
  fetchHandler,
  doValid,
  defineRequest,
  createGlobalClient,
  createClient,
  cloneClient,
  basicAuthInterceptor,
  ERR_UNSUPPORTED_FIELD_TYPE,
  ERR_UNKNOWN,
  ERR_TIMEOUT,
  ERR_OBSERVE,
  ERR_NOT_SET_ALIAS,
  ERR_NOT_FOUND_HANDLER,
  ERR_NOT_FOUND_GLOBAL_CLIENT,
  ERR_NETWORK,
  ERR_INVALID_HTTP_CONTEXT_TOKEN,
  ERR_INVALID_CLIENT,
  ERR_ABORTED
};
